@@ -74,10 +74,11 @@ void main_loop_worker_job_complete(void);
 
 void main_loop_worker_thread_start(void *cookie);
 void main_loop_worker_thread_stop(void);
+void main_loop_worker_run_gc(void);
 
 void main_loop_create_worker_thread(WorkerThreadFunc func, WorkerExitNotificationFunc terminate_func, gpointer data, WorkerOptions *worker_options);
 
-void main_loop_worker_sync_call(void (*func)(void));
+void main_loop_worker_sync_call(void (*func)(void *user_data), void *user_data);
 
 void main_loop_worker_init(void);
 void main_loop_worker_deinit(void);
